﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.PackingEncoding.PackingSequence {
  [Item("PackingSequenceCreator", "Creator class used to create PackingSequence solutions for bin packing problems.")]
  [StorableClass]
  public class PackingSequenceRandomCreator : PackingSolutionCreator, IPackingSequenceCreator, IStochasticOperator {

    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<IntValue> NumberOfPackingItemsParameter {
      get { return (ILookupParameter<IntValue>)Parameters["NumberOfItems"]; }
    }

    public ILookupParameter<Permutation> SolutionParameter {
      get { return (ILookupParameter<Permutation>)Parameters["PackingSequence"]; }
    }

    [StorableConstructor]
    protected PackingSequenceRandomCreator(bool deserializing) : base(deserializing) { }
    protected PackingSequenceRandomCreator(PackingSequenceRandomCreator original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new PackingSequenceRandomCreator(this, cloner);
    }

    public PackingSequenceRandomCreator()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator."));
      Parameters.Add(new ValueLookupParameter<IntValue>("NumberOfPackingItems", "The number of packing-items handled in this problem instance."));
      Parameters.Add(new LookupParameter<Permutation>("PackingSequence", "The randomly created solution candidate."));
    }

    public override IOperation Apply() {
      Apply(NumberOfPackingItemsParameter.ActualValue.Value, RandomParameter.ActualValue);
      return base.Apply();
    }

    public static Permutation Apply(int items, IRandom random) {
      return new Permutation(PermutationTypes.Absolute, items, random);
    }
    protected override IItem CreateSolution() {
      return Apply(NumberOfPackingItemsParameter.ActualValue.Value, RandomParameter.ActualValue);
    }
  }
}
